<!--
// © University of Southampton IT Innovation Centre, 2017
//
// Copyright in this software belongs to University of Southampton
// IT Innovation Centre of Gamma House, Enterprise Road, 
// Chilworth Science Park, Southampton, SO16 7NS, UK.
//
// This software may not be used, sold, licensed, transferred, copied
// or reproduced in whole or in part in any manner or form or in or
// on any media by any person other than in accordance with the terms
// of the Licence Agreement supplied with the software, or otherwise
// without the prior written consent of the copyright owners.
//
// This software is distributed WITHOUT ANY WARRANTY, without even the
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
// PURPOSE, except where stated in the Licence Agreement supplied with
// the software.
//
//      Created By :            Michael Boniface
//      Updated By :            Simon Crowle
//      Created Date :          18-12-2017
//      Update Date :           14-02-2018
//      Created for Project :   FLAME
-->

# CLMC Adaptive Streaming Test

This test streams mpeg-dash video using the two apache servers monitored by Telegraf configured with a default apache plugin and a net_response plugin. The data is stored in the `clmc-service` using database `CLMCMetrics` and measurements `apache` and `net_response`

The following command brings up the services

`vagrant --infra=streaming up`

* clmc-service: configured with influx, kapacitor, chornograf
* apache1@DC1, apache2@DC2: configured with apache and a test video located at http://192.168.50.11:80/test_video/stream.mpd on the internal vbox network and at http://localhost:8081/test_video/stream.mpd if accessing from the host machine

## Automatic set-up of the CLMC dashboard demonstrator
To automatically set up a demonstrator CLMC dashboard and Kapacitor, run the following:

```
cd ./test/streaming/
./config-dashboard.sh
```

And then point your browser to:

`http://localhost:8888`

## Manual set-up of Chronograf's CLMC data source

If you __do not__ want to run the automatic set-up, basic entry to the Chronograf dashboard is as follows:

1. Point your browser to: [http://localhost:8888](http://localhost:8888)
2. Enter your connection string: `http://localhost:8086`
3. Enter the Name: `Influx 1`
4. Enter the Telegraf database: `CLMCMetrics`

## Manual test on Windows

### View the video
Install VLC video client on the host machine, you must use a very recent version otherwise the MPD file cannot we read. At the time of writng the following nighly build was installed:

https://nightlies.videolan.org/build/win32/vlc-3.0.0-rc1-20171201-0326/vlc-3.0.0-20171201-0326-rc1-win32.exe

Start the VLC Player

`Media->Open Network Stream`

The test video is the FLAME project video and it can be viewed at the following location.

`Enter the network URL: http://localhost:8081/test-video/stream.mpd for apache1 server`

The video should play.

### Query the data

Open Chronograph by entering the following URL into a browser on the host http://localhost:8888. Your CLMC data source, Kapacitor and demonstration dashboard should be ready for you to explore.

Press the Data Explorer in the menu and select the apache measurement and create a query such as 

`SELECT mean("BytesPerSec") AS "mean_BytesPerSec" FROM "CLMCMetrics"."autogen"."apache" WHERE time > now() - 5m GROUP BY time(10s)`

## KPI triggers

In this demonstrator an example KPI rule has been set up in Kapacitor which fires when the average requests per second on the Apache 1 server goes above certain thresholds ( a 'warning' at 0.2 requests/second and a 'critical' message at 0.5 requests/second ). The TICKscript specification for this rule is as follows:

```
dbrp "CLMCMetrics"."autogen"

var data = batch
    |query(''' SELECT mean("ReqPerSec") AS "mean_RPS" FROM "CLMCMetrics"."autogen"."apache" WHERE "ipendpoint"='adaptive_streaming_I1_apache1' ''')
        .period(5s)
        .every(5s)

var alert = data
    |alert()
        .id('{{ .Name }}/adaptive_streaming_I1_apache1')
        .message('{{ .ID }} is {{ .Level }} Mean Requests Per Second: {{ index .Fields "mean_RPS" }}')
        .warn(lambda: "mean_RPS" > 0.2)
        .crit(lambda: "mean_RPS" > 0.5)
        .slack()
        .log( '/tmp/RPSLoad.log' )
```

Alerts are sent both an internal logging within the CLMC service file system and also to a FLAME demo Slack service:

https://flamedemo-itinnov.slack.com

Alerts can be found under the '#clmc' channel. 

### Kapacitor rules in Chronograf's GUI

Additional rules can be added to this demonstrator either via the Chronograf GUI (see [here](https://docs.influxdata.com/chronograf/v1.4/introduction/getting-started/#4-connect-chronograf-to-kapacitor) for more information) or by using the Kapacitor HTTP API and TICKscript (for an introduction, [look here](https://docs.influxdata.com/kapacitor/v1.4/tick/)).
 
## Automatic test using JMeter and VLC command line

To be completed.

* Create a VM called `vlc-client` with vlc and jmeter installed
* Write a testcase using the JMeter GUI that streams the test video to a set of clients.
  * The options for scaling the number of client threads needs to be explored
  * The test conditions needs to be defined i.e. triggers at specific thresholds when cpu usage reaches a certain point

Issues, the Vagrantfile needs updating as the `vlc-client` needs to run two provisioning scripts adn the assumption is there's one provisioning script and the telegraf provisioning script. Looks like a hack is coming....if `vlc-client` then provision jmeter.

We then need programtically create rules using the kapacitor CLI to set the KPIs and get the alerts rather than all the manual set up 